# *****************************************************************************
#
# Copyright (c) 2020, the pyEX authors.
#
# This file is part of the jupyterlab_templates library, distributed under the terms of
# the Apache License 2.0.  The full license can be found in the LICENSE file.
#
from .client import *  # noqa: F403
from .common import PyEXception, PyEXStopSSE, overrideSSEUrl, overrideUrl, setProxy

try:
    from .studies import *  # noqa: F403
except ImportError:
    pass

from ._version import __version__
from .account import *  # noqa: F403
from .alternative import (
    ceoCompensation,
    ceoCompensationDF,
    crypto,
    cryptoDF,
    sentiment,
    sentimentDF,
)
from .commodities import CommoditiesPoints
from .cryptocurrency import (
    cryptoBook,
    cryptoBookDF,
    cryptoPrice,
    cryptoPriceDF,
    cryptoQuote,
    cryptoQuoteDF,
)
from .economic import EconomicPoints
from .fx import (
    convertFX,
    convertFXDF,
    historicalFX,
    historicalFXDF,
    latestFX,
    latestFXDF,
)
from .marketdata.cryptocurrency import (
    cryptoBookSSE,
    cryptoBookSSEAsync,
    cryptoEventsSSE,
    cryptoEventsSSEAsync,
    cryptoQuotesSSE,
    cryptoQuotesSSEAsync,
)
from .marketdata.fx import (
    forex1MinuteSSE,
    forex1MinuteSSEAsync,
    forex1SecondSSE,
    forex1SecondSSEAsync,
    forex5SecondSSE,
    forex5SecondSSEAsync,
    fxSSE,
    fxSSEAsync,
)
from .marketdata.http import auction, auctionAsync, auctionDF
from .marketdata.http import book as deepBook
from .marketdata.http import bookAsync as deepBookAsync
from .marketdata.http import bookDF as deepBookDF
from .marketdata.http import (
    deep,
    deepAsync,
    deepDF,
    hist,
    histAsync,
    histDF,
    last,
    lastAsync,
    lastDF,
    officialPrice,
    officialPriceAsync,
    officialPriceDF,
    opHaltStatus,
    opHaltStatusAsync,
    opHaltStatusDF,
    securityEvent,
    securityEventAsync,
    securityEventDF,
    ssrStatus,
    ssrStatusAsync,
    ssrStatusDF,
    systemEvent,
    systemEventAsync,
    systemEventDF,
    tops,
    topsAsync,
    topsDF,
    tradeBreak,
    tradeBreakAsync,
    tradeBreakDF,
    trades,
    tradesAsync,
    tradesDF,
    tradingStatus,
    tradingStatusAsync,
    tradingStatusDF,
)
from .marketdata.news import newsSSE, newsSSEAsync
from .marketdata.sentiment import sentimentSSE, sentimentSSEAsync
from .marketdata.sse import (
    auctionSSE,
    auctionSSEAsync,
    bookSSE,
    bookSSEAsync,
    deepSSE,
    deepSSEAsync,
    lastSSE,
    lastSSEAsync,
    officialPriceSSE,
    officialPriceSSEAsync,
    opHaltStatusSSE,
    opHaltStatusSSEAsync,
    securityEventSSE,
    securityEventSSEAsync,
    ssrStatusSSE,
    ssrStatusSSEAsync,
    systemEventSSE,
    systemEventSSEAsync,
    topsSSE,
    topsSSEAsync,
    tradeBreaksSSE,
    tradeBreaksSSEAsync,
    tradesSSE,
    tradesSSEAsync,
    tradingStatusSSE,
    tradingStatusSSEAsync,
)
from .marketdata.stock import (
    stocksUS1MinuteSSE,
    stocksUS1MinuteSSEAsync,
    stocksUS1SecondSSE,
    stocksUS1SecondSSEAsync,
    stocksUS5SecondSSE,
    stocksUS5SecondSSEAsync,
    stocksUSNoUTPSSE,
    stocksUSNoUTPSSEAsync,
    stocksUSSSE,
    stocksUSSSEAsync,
)
from .marketdata.ws import *  # noqa: F403
from .markets import markets, marketsDF
from .options import optionExpirations, options, optionsDF
from .points import points, pointsDF
from .premium import (
    accountingQualityAndRiskMatrix,
    accountingQualityAndRiskMatrixDF,
    analystDays,
    analystDaysDF,
    boardOfDirectorsMeeting,
    boardOfDirectorsMeetingDF,
    brain2DayMLReturnRanking,
    brain2DayMLReturnRankingDF,
    brain3DayMLReturnRanking,
    brain3DayMLReturnRankingDF,
    brain5DayMLReturnRanking,
    brain5DayMLReturnRankingDF,
    brain7DaySentiment,
    brain7DaySentimentDF,
    brain10DayMLReturnRanking,
    brain10DayMLReturnRankingDF,
    brain21DayMLReturnRanking,
    brain21DayMLReturnRankingDF,
    brain30DaySentiment,
    brain30DaySentimentDF,
    brainLanguageMetricsOnCompanyFilings,
    brainLanguageMetricsOnCompanyFilingsAll,
    brainLanguageMetricsOnCompanyFilingsAllDF,
    brainLanguageMetricsOnCompanyFilingsDF,
    brainLanguageMetricsOnCompanyFilingsDifference,
    brainLanguageMetricsOnCompanyFilingsDifferenceAll,
    brainLanguageMetricsOnCompanyFilingsDifferenceAllDF,
    brainLanguageMetricsOnCompanyFilingsDifferenceDF,
    businessUpdates,
    businessUpdatesDF,
    buybacks,
    buybacksDF,
    cam1,
    cam1DF,
    capitalMarketsDay,
    capitalMarketsDayDF,
    companyTravel,
    companyTravelDF,
    directorAndOfficerChanges,
    directorAndOfficerChangesDF,
    esgCFPBComplaints,
    esgCFPBComplaintsDF,
    esgCPSCRecalls,
    esgCPSCRecallsDF,
    esgDOLVisaApplications,
    esgDOLVisaApplicationsDF,
    esgEPAEnforcements,
    esgEPAEnforcementsDF,
    esgEPAMilestones,
    esgEPAMilestonesDF,
    esgFECIndividualCampaingContributions,
    esgFECIndividualCampaingContributionsDF,
    esgOSHAInspections,
    esgOSHAInspectionsDF,
    esgSenateLobbying,
    esgSenateLobbyingDF,
    esgUSASpending,
    esgUSASpendingDF,
    esgUSPTOPatentApplications,
    esgUSPTOPatentApplicationsDF,
    esgUSPTOPatentGrants,
    esgUSPTOPatentGrantsDF,
    fdaAdvisoryCommitteeMeetings,
    fdaAdvisoryCommitteeMeetingsDF,
    filingDueDates,
    filingDueDatesDF,
    fiscalQuarterEnd,
    fiscalQuarterEndDF,
    forum,
    forumDF,
    generalConference,
    generalConferenceDF,
    holidaysWSH,
    holidaysWSHDF,
    indexChanges,
    indexChangesDF,
    iposWSH,
    iposWSHDF,
    kScore,
    kScoreChina,
    kScoreChinaDF,
    kScoreDF,
    legalActions,
    legalActionsDF,
    mergersAndAcquisitions,
    mergersAndAcquisitionsDF,
    nonTimelyFilings,
    nonTimelyFilingsDF,
    precisionAlphaPriceDynamics,
    precisionAlphaPriceDynamicsDF,
    productEvents,
    productEventsDF,
    researchAndDevelopmentDays,
    researchAndDevelopmentDaysDF,
    sameStoreSales,
    sameStoreSalesDF,
    secondaryOfferings,
    secondaryOfferingsDF,
    seminars,
    seminarsDF,
    shareholderMeetings,
    shareholderMeetingsDF,
    similarityIndex,
    similarityIndexDF,
    summitMeetings,
    summitMeetingsDF,
    tacticalModel1,
    tacticalModel1DF,
    tradeShows,
    tradeShowsDF,
    valuEngineStockResearchReport,
    witchingHours,
    witchingHoursDF,
    workshops,
    workshopsDF,
)
from .rates import RatesPoints
from .refdata import (
    calendar,
    calendarDF,
    corporateActions,
    corporateActionsDF,
    cryptoSymbols,
    cryptoSymbolsDF,
    cryptoSymbolsList,
    directory,
    directoryDF,
    exchanges,
    exchangesDF,
    fxSymbols,
    fxSymbolsDF,
    fxSymbolsList,
    holidays,
    holidaysDF,
    iexSymbols,
    iexSymbolsDF,
    iexSymbolsList,
    internationalExchanges,
    internationalExchangesDF,
    internationalSymbols,
    internationalSymbolsDF,
    internationalSymbolsList,
    mutualFundSymbols,
    mutualFundSymbolsDF,
    mutualFundSymbolsList,
    nextDayExtDate,
    nextDayExtDateDF,
    optionsSymbols,
    optionsSymbolsDF,
    optionsSymbolsList,
    otcSymbols,
    otcSymbolsDF,
    otcSymbolsList,
    refDividends,
    refDividendsDF,
    search,
    searchDF,
    sectors,
    sectorsDF,
    symbols,
    symbolsDF,
    symbolsList,
    tags,
    tagsDF,
)
from .rules import create, delete, lookup
from .rules import output as ruleOutput
from .rules import pause, resume
from .rules import rule as ruleInfo
from .rules import rules, schema
from .stats import (
    daily,
    dailyDF,
    recent,
    recentDF,
    records,
    recordsDF,
    stats,
    statsDF,
    summary,
    summaryDF,
)
from .stocks import (
    advancedStats,
    advancedStatsDF,
    analystRecommendations,
    analystRecommendationsDF,
    balanceSheet,
    balanceSheetDF,
    batch,
    batchDF,
    bonusIssue,
    bonusIssueDF,
    book,
    bookDF,
    bulkBatch,
    bulkBatchDF,
    bulkMinuteBars,
    bulkMinuteBarsDF,
    cashFlow,
    cashFlowDF,
    chart,
    chartDF,
    collections,
    collectionsDF,
    company,
    companyDF,
    delayedQuote,
    delayedQuoteDF,
    distribution,
    distributionDF,
    dividends,
    dividendsBasic,
    dividendsBasicDF,
    dividendsDF,
    earnings,
    earningsDF,
    earningsToday,
    earningsTodayDF,
    estimates,
    estimatesDF,
    financials,
    financialsDF,
    fundamentals,
    fundamentalsDF,
    fundOwnership,
    fundOwnershipDF,
    incomeStatement,
    incomeStatementDF,
    insiderRoster,
    insiderRosterDF,
    insiderSummary,
    insiderSummaryDF,
    insiderTransactions,
    insiderTransactionsDF,
    institutionalOwnership,
    institutionalOwnershipDF,
    intraday,
    intradayDF,
    ipoToday,
    ipoTodayDF,
    ipoUpcoming,
    ipoUpcomingDF,
    keyStats,
    keyStatsDF,
    largestTrades,
    largestTradesDF,
    list,
    listDF,
    logo,
    logoNotebook,
    logoPNG,
    marketNews,
    marketNewsDF,
    marketOhlc,
    marketOhlcDF,
    marketPrevious,
    marketPreviousDF,
    marketShortInterest,
    marketShortInterestDF,
    marketVolume,
    marketVolumeDF,
    marketYesterday,
    marketYesterdayDF,
    news,
    newsDF,
    ohlc,
    ohlcDF,
    peers,
    peersDF,
    previous,
    previousDF,
    price,
    priceDF,
    priceTarget,
    priceTargetDF,
    quote,
    quoteDF,
    relevant,
    relevantDF,
    returnOfCapital,
    returnOfCapitalDF,
    rightsIssue,
    rightsIssueDF,
    rightToPurchase,
    rightToPurchaseDF,
    sectorPerformance,
    sectorPerformanceDF,
    securityReclassification,
    securityReclassificationDF,
    securitySwap,
    securitySwapDF,
    shortInterest,
    shortInterestDF,
    spinoff,
    spinoffDF,
    splits,
    splitsDF,
    spread,
    spreadDF,
    stockSplits,
    stockSplitsDF,
    threshold,
    thresholdDF,
    upcomingDividends,
    upcomingDividendsDF,
    upcomingEarnings,
    upcomingEarningsDF,
    upcomingEvents,
    upcomingEventsDF,
    upcomingIPOs,
    upcomingIPOsDF,
    upcomingSplits,
    upcomingSplitsDF,
    volumeByVenue,
    volumeByVenueDF,
    yesterday,
    yesterdayDF,
)

try:
    from .caching import *  # noqa: F403
except ImportError:
    pass

try:
    from .zipline import *  # noqa: F403
except ImportError:
    pass
